package org.jboss.example;

import javax.annotation.Resource;
import javax.inject.Inject;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSContext;
import javax.jms.JMSDestinationDefinition;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.QueryParam;

@Path("jms")
@JMSDestinationDefinition(interfaceName = "javax.jms.Queue", name = "java:/java-testQueue")
public class Messaging {

	@Resource(name = "ConnectionFactory")
	ConnectionFactory cf;
	
	@Resource(mappedName = "java:/java-testQueue")
	Queue testQueue;
	
	@Inject
	JMSContext ctx;
	
	@GET
	@Path("old/send")
	public String sendOld(@QueryParam("msg") String msg) throws Exception {
		Connection conn = null;
		Session session = null;
		MessageProducer producer = null;
		try {
			conn = cf.createConnection();
			conn.start();
			session = conn.createSession(Session.AUTO_ACKNOWLEDGE);
			producer = session.createProducer(testQueue);
			TextMessage jmsMsg = session.createTextMessage();
			jmsMsg.setText(msg);
			producer.send(jmsMsg);
		}
		finally {
			if (producer != null) {
				try {
					producer.close();
				} catch (JMSException e) {
				}
			}
			if (session != null) {
				try {
					session.close();
				} catch (JMSException e) {
				}
			}
			if (conn != null) {
				try {
					conn.close();
				} catch (JMSException e) {
				}
			}
		}
		return "OK";
	}
	
	@GET
	@Path("new/send")
	public String sendNew(@QueryParam("msg") String msg) throws Exception {
		ctx.createProducer().send(testQueue, ctx.createTextMessage(msg));
		return "OK";
	}
	
	@GET
	@Path("old/recv")
	public String receiveOld() throws Exception {
		try (Connection conn = cf.createConnection();
				Session session = conn.createSession(Session.AUTO_ACKNOWLEDGE);
				MessageConsumer consumer = session.createConsumer(testQueue)) {
			conn.start();
			Message jmsMsg = consumer.receiveNoWait();
			String msg = "NULL";
			if (jmsMsg instanceof TextMessage) {
				msg = ((TextMessage) jmsMsg).getText();
			}
			return msg;
		}
	}
	
	@GET
	@Path("new/recv")
	public String receiveNew() throws Exception {
		String msg = ctx.createConsumer(testQueue).receiveBodyNoWait(String.class);
		return msg == null ? "NULL" : msg;
	}
}
